<?php


namespace app\admin\controller;


use think\App;use think\facade\Validate;
class Result extends Base{

    public function __construct(App $app) {
        parent::__construct($app);
        $this->model = new \app\admin\model\ResultInfo();
    }

    public function create(){
         $param = $this->request->param(["result"=>"","result_desc"=>"","type"=>"","status"=>0],"post","trim");
         $valid=Validate::rule([
             "result|异常原因"=>"require",
             "result_desc|异常描述"=>"require",
             "type|类型"=>"require|number",
             "status|状态"=>"require|number|in:0,1",
         ]);
         if(!$valid->check($param)) return error($valid->getError());
         $num = $this->model->count();
         $str = sprintf("%04d",$num);
         $data=[
             "result"=>$param['result'],
             "result_desc"=>$param['result_desc'],
             "result_code"=>$str,
             "type"=>$param['type'],
             "status"=>$param['status'],
         ];
         $create = $this->model->create($data);
         if($create->isEmpty())return error("添加失败");
         return success("添加成功");
    }

    public function save(){
        $param = $this->request->param(["id"=>"","result"=>"","result_desc"=>"","type"=>"","status"=>0],"post","trim");
        $valid=Validate::rule([
            "id|id"=>"require|number",
            "result|异常原因"=>"require",
            "result_desc|异常描述"=>"require",
            "type|类型"=>"require|number",
            "status|状态"=>"require|number|in:0,1",
        ]);
        if(!$valid->check($param)) return error($valid->getError());
        $info = $this->model->findOrEmpty($param['id']);
        if($info->isEmpty())return error("未找到该记录");
        $info->result = $param['result'];
        $info->result_desc = $param['result_desc'];
        $info->type = $param['type'];
        $info->status = $param['status'];
        if(!$info->save())return error("保存失败");
        return success("保存成功");
    }

    public function delete(){
        $param = $this->request->param(["id"=>""],"post","trim");
        $valid=Validate::rule([
            "id|id"=>"require|number",
        ]);
        if(!$valid->check($param)) return error($valid->getError());
        $info = $this->model->findOrEmpty($param['id']);
        if($info->isEmpty())return error("未找到该记录");
        $info->is_del = 1;
        if(!$info->save())return error("删除失败");
        return success("删除成功");
    }

    public function list(){
        $param = $this->request->param(["page"=>1,"size"=>10,"type"=>""],"post","trim");
        $where=[["is_del","=",0]];
        if($param['type']!=="") $where[]=["type","=",$param['type']];
        $list = $this->model->where($where)->order("addtime desc")
        ->paginate(["page"=>$param['page'],"list_rows"=>$param['size']]);
        return success("获取成功",["list"=>$list->items(),"count"=>$list->total()]);
    }

    public function status(){
        $param = $this->request->param(["id"=>"","status"=>0],"post","trim");
        $valid=Validate::rule([
            "id|id"=>"require|number",
            "status|状态"=>"require|number|in:0,1",
        ]);
        if(!$valid->check($param)) return error($valid->getError());
        $info = $this->model->where('is_del','=',0)->findOrEmpty($param['id']);
        if($info->isEmpty())return error("未找到该记录");
        $info->status = $param['status'];
        if(!$info->save())return error("保存失败");
        return success("保存成功");
    }
    public function query(){
        $param = $this->request->param(["status"=>""],"post","trim");
        $where=[["is_del","=",0]];
        if($param['status']!=="") $where[]=["status","=",$param['status']];
        $info = $this->model->where($where)->order("id desc")->select();
        return success("获取成功",$info);
    }
}